@@ -8,16 +8,21 @@
 #include "gnome-settings-locate-pointer.h"
 #include "gnome-settings-daemon.h"
 
-#define MAX_BUTTONS 10
+#define DEFAULT_PTR_MAP_SIZE 128
 
 static void
 set_left_handed (gboolean left_handed)
 {
-  unsigned char buttons[MAX_BUTTONS];
+  unsigned char *buttons;
   gint n_buttons, i;
   gint idx_1 = 0, idx_3 = 1;
 
-  n_buttons = XGetPointerMapping (GDK_DISPLAY (), buttons, MAX_BUTTONS);
+  buttons = g_alloca (DEFAULT_PTR_MAP_SIZE);
+  n_buttons = XGetPointerMapping (GDK_DISPLAY (), buttons, DEFAULT_PTR_MAP_SIZE);
+  if (n_buttons > DEFAULT_PTR_MAP_SIZE) {
+    buttons = g_alloca (n_buttons);
+    n_buttons = XGetPointerMapping (GDK_DISPLAY (), buttons, n_buttons);
+  }
 
   for (i = 0; i < n_buttons; i++)
     {
@@ -78,7 +83,7 @@ set_motion_acceleration (gfloat motion_acceleration)
       numerator = -1;
       denominator = -1;
     }
-  g_print ("%d/%d\n", numerator, denominator);
+
   XChangePointerControl (GDK_DISPLAY (), True, False,
 			 numerator, denominator,
 			 0);
@@ -116,7 +121,7 @@ filter (GdkXEvent *xevent,
 					   group,
 					   &keyval,
 					   NULL, NULL, NULL);
-      if (keyval == GDK_Control_L)
+      if (keyval == GDK_Control_L || keyval == GDK_Control_R)
 	{
 	  if (xev->type == KeyPress)
 	    {
@@ -152,37 +157,41 @@ set_locate_pointer (gboolean locate_pointer)
   GdkKeymapKey *keys;
   int n_keys;
   gboolean has_entries;
+  static const guint keyvals[] = { GDK_Control_L, GDK_Control_R };
+  unsigned j;
 
-  has_entries = gdk_keymap_get_entries_for_keyval (gdk_keymap_get_default (),
-						   GDK_Control_L,
-						   &keys,
-						   &n_keys);
-  if (has_entries)
+  for (j = 0 ; j < G_N_ELEMENTS (keyvals) ; j++) {
+    has_entries = gdk_keymap_get_entries_for_keyval (gdk_keymap_get_default (),
+							   keyvals[j],
+							   &keys,
+							   &n_keys);
+    if (has_entries)
     {
       gint i;
 
       for (i = 0; i < n_keys; i++)
-	{
-	  if (locate_pointer)
-	    XGrabKey (gdk_x11_get_default_xdisplay (),
+      {
+	     if (locate_pointer)
+	       XGrabKey (gdk_x11_get_default_xdisplay (),
 		      keys[i].keycode,
 		      AnyModifier,
 		      GDK_ROOT_WINDOW (),
 		      False,
 		      GrabModeAsync,
 		      GrabModeSync);
-	  else
-	    XUngrabKey (gdk_x11_get_default_xdisplay (),
-			keys[i].keycode,
-			AnyModifier,
-			GDK_ROOT_WINDOW ());
-	}
-      g_free (keys);
-      if (locate_pointer)
-	gdk_window_add_filter (gdk_get_default_root_window (), filter, NULL);
-      else
-	gdk_window_remove_filter (gdk_get_default_root_window (), filter, NULL);
+	     else
+	       XUngrabKey (gdk_x11_get_default_xdisplay (),
+		      keys[i].keycode,
+		      AnyModifier,
+		      GDK_ROOT_WINDOW ());
+     }
+     g_free (keys);
+     if (locate_pointer)
+	    gdk_window_add_filter (gdk_get_default_root_window (), filter, NULL);
+    else
+	    gdk_window_remove_filter (gdk_get_default_root_window (), filter, NULL);
     }
+  }
 }
 
 static void
